@@ -395,11 +395,12 @@ class MultiOutputClassifier(ClassifierMixin, _MultiOutputEstimator):
     >>> import numpy as np
     >>> from sklearn.datasets import make_multilabel_classification
     >>> from sklearn.multioutput import MultiOutputClassifier
-    >>> from sklearn.neighbors import KNeighborsClassifier
+    >>> from sklearn.linear_model import LogisticRegression
     >>> X, y = make_multilabel_classification(n_classes=3, random_state=0)
-    >>> clf = MultiOutputClassifier(KNeighborsClassifier()).fit(X, y)
+    >>> clf = MultiOutputClassifier(LogisticRegression()).fit(X, y)
     >>> clf.predict(X[-2:])
-    array([[1, 1, 0], [1, 1, 1]])
+    array([[1, 1, 1],
+           [1, 0, 1]])
     """
 
     def __init__(self, estimator, *, n_jobs=None):
@@ -852,7 +853,7 @@ class RegressorChain(MetaEstimatorMixin, RegressorMixin, _BaseChain):
     Parameters
     ----------
     base_estimator : estimator
-        The base estimator from which the classifier chain is built.
+        The base estimator from which the regressor chain is built.
 
     order : array-like of shape (n_outputs,) or 'random', default=None
         If `None`, the order will be determined by the order of columns in
